// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/compute/disks/v1/disks.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_COMPUTE_DISKS_V1_DISKS_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_COMPUTE_DISKS_V1_DISKS_CONNECTION_H

#include "google/cloud/compute/disks/v1/disks_connection_idempotency_policy.h"
#include "google/cloud/compute/disks/v1/internal/disks_retry_traits.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/experimental_tag.h"
#include "google/cloud/future.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/compute/disks/v1/disks.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace compute_disks_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `DisksConnection`.
class DisksRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<DisksRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `DisksConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class DisksLimitedErrorCountRetryPolicy : public DisksRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit DisksLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  DisksLimitedErrorCountRetryPolicy(
      DisksLimitedErrorCountRetryPolicy&& rhs) noexcept
      : DisksLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}
  DisksLimitedErrorCountRetryPolicy(
      DisksLimitedErrorCountRetryPolicy const& rhs) noexcept
      : DisksLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<DisksRetryPolicy> clone() const override {
    return std::make_unique<DisksLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = DisksRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      compute_disks_v1_internal::DisksRetryTraits>
      impl_;
};

/**
 * A retry policy for `DisksConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class DisksLimitedTimeRetryPolicy : public DisksRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit DisksLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  DisksLimitedTimeRetryPolicy(DisksLimitedTimeRetryPolicy&& rhs) noexcept
      : DisksLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  DisksLimitedTimeRetryPolicy(DisksLimitedTimeRetryPolicy const& rhs) noexcept
      : DisksLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<DisksRetryPolicy> clone() const override {
    return std::make_unique<DisksLimitedTimeRetryPolicy>(maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = DisksRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      compute_disks_v1_internal::DisksRetryTraits>
      impl_;
};

/**
 * The `DisksConnection` object for `DisksClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `DisksClient`. This allows users to inject custom behavior
 * (e.g., with a Google Mock object) when writing tests that use objects of type
 * `DisksClient`.
 *
 * To create a concrete instance, see `MakeDisksConnection()`.
 *
 * For mocking, see `compute_disks_v1_mocks::MockDisksConnection`.
 */
class DisksConnection {
 public:
  virtual ~DisksConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  AddResourcePolicies(
      google::cloud::cpp::compute::disks::v1::AddResourcePoliciesRequest const&
          request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  AddResourcePolicies(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::AddResourcePoliciesRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  AddResourcePolicies(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StreamRange<
      std::pair<std::string, google::cloud::cpp::compute::v1::DisksScopedList>>
  AggregatedListDisks(
      google::cloud::cpp::compute::disks::v1::AggregatedListDisksRequest
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  BulkInsert(
      google::cloud::cpp::compute::disks::v1::BulkInsertRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> BulkInsert(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::BulkInsertRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  BulkInsert(ExperimentalTag,
             google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  CreateSnapshot(
      google::cloud::cpp::compute::disks::v1::CreateSnapshotRequest const&
          request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> CreateSnapshot(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::CreateSnapshotRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  CreateSnapshot(ExperimentalTag,
                 google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  DeleteDisk(
      google::cloud::cpp::compute::disks::v1::DeleteDiskRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> DeleteDisk(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::DeleteDiskRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  DeleteDisk(ExperimentalTag,
             google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StatusOr<google::cloud::cpp::compute::v1::Disk> GetDisk(
      google::cloud::cpp::compute::disks::v1::GetDiskRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Policy> GetIamPolicy(
      google::cloud::cpp::compute::disks::v1::GetIamPolicyRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  InsertDisk(
      google::cloud::cpp::compute::disks::v1::InsertDiskRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> InsertDisk(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::InsertDiskRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  InsertDisk(ExperimentalTag,
             google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StreamRange<google::cloud::cpp::compute::v1::Disk> ListDisks(
      google::cloud::cpp::compute::disks::v1::ListDisksRequest request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  RemoveResourcePolicies(google::cloud::cpp::compute::disks::v1::
                             RemoveResourcePoliciesRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  RemoveResourcePolicies(ExperimentalTag, NoAwaitTag,
                         google::cloud::cpp::compute::disks::v1::
                             RemoveResourcePoliciesRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  RemoveResourcePolicies(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>> Resize(
      google::cloud::cpp::compute::disks::v1::ResizeRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> Resize(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::ResizeRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>> Resize(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StatusOr<google::cloud::cpp::compute::v1::Policy> SetIamPolicy(
      google::cloud::cpp::compute::disks::v1::SetIamPolicyRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  SetLabels(
      google::cloud::cpp::compute::disks::v1::SetLabelsRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> SetLabels(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::SetLabelsRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  SetLabels(ExperimentalTag,
            google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  StartAsyncReplication(google::cloud::cpp::compute::disks::v1::
                            StartAsyncReplicationRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  StartAsyncReplication(ExperimentalTag, NoAwaitTag,
                        google::cloud::cpp::compute::disks::v1::
                            StartAsyncReplicationRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  StartAsyncReplication(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  StopAsyncReplication(
      google::cloud::cpp::compute::disks::v1::StopAsyncReplicationRequest const&
          request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  StopAsyncReplication(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::StopAsyncReplicationRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  StopAsyncReplication(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  StopGroupAsyncReplication(
      google::cloud::cpp::compute::disks::v1::
          StopGroupAsyncReplicationRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  StopGroupAsyncReplication(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::
          StopGroupAsyncReplicationRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  StopGroupAsyncReplication(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StatusOr<google::cloud::cpp::compute::v1::TestPermissionsResponse>
  TestIamPermissions(
      google::cloud::cpp::compute::disks::v1::TestIamPermissionsRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  UpdateDisk(
      google::cloud::cpp::compute::disks::v1::UpdateDiskRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> UpdateDisk(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::disks::v1::UpdateDiskRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  UpdateDisk(ExperimentalTag,
             google::cloud::cpp::compute::v1::Operation const& operation);
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace compute_disks_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_COMPUTE_DISKS_V1_DISKS_CONNECTION_H
