// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/compute/target_pools/v1/target_pools.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_COMPUTE_TARGET_POOLS_V1_TARGET_POOLS_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_COMPUTE_TARGET_POOLS_V1_TARGET_POOLS_CONNECTION_H

#include "google/cloud/compute/target_pools/v1/internal/target_pools_retry_traits.h"
#include "google/cloud/compute/target_pools/v1/target_pools_connection_idempotency_policy.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/experimental_tag.h"
#include "google/cloud/future.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/compute/target_pools/v1/target_pools.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace compute_target_pools_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `TargetPoolsConnection`.
class TargetPoolsRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<TargetPoolsRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `TargetPoolsConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class TargetPoolsLimitedErrorCountRetryPolicy : public TargetPoolsRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit TargetPoolsLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  TargetPoolsLimitedErrorCountRetryPolicy(
      TargetPoolsLimitedErrorCountRetryPolicy&& rhs) noexcept
      : TargetPoolsLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}
  TargetPoolsLimitedErrorCountRetryPolicy(
      TargetPoolsLimitedErrorCountRetryPolicy const& rhs) noexcept
      : TargetPoolsLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<TargetPoolsRetryPolicy> clone() const override {
    return std::make_unique<TargetPoolsLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = TargetPoolsRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      compute_target_pools_v1_internal::TargetPoolsRetryTraits>
      impl_;
};

/**
 * A retry policy for `TargetPoolsConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class TargetPoolsLimitedTimeRetryPolicy : public TargetPoolsRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit TargetPoolsLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  TargetPoolsLimitedTimeRetryPolicy(
      TargetPoolsLimitedTimeRetryPolicy&& rhs) noexcept
      : TargetPoolsLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  TargetPoolsLimitedTimeRetryPolicy(
      TargetPoolsLimitedTimeRetryPolicy const& rhs) noexcept
      : TargetPoolsLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<TargetPoolsRetryPolicy> clone() const override {
    return std::make_unique<TargetPoolsLimitedTimeRetryPolicy>(
        maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = TargetPoolsRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      compute_target_pools_v1_internal::TargetPoolsRetryTraits>
      impl_;
};

/**
 * The `TargetPoolsConnection` object for `TargetPoolsClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `TargetPoolsClient`. This allows users to inject custom behavior
 * (e.g., with a Google Mock object) when writing tests that use objects of type
 * `TargetPoolsClient`.
 *
 * To create a concrete instance, see `MakeTargetPoolsConnection()`.
 *
 * For mocking, see `compute_target_pools_v1_mocks::MockTargetPoolsConnection`.
 */
class TargetPoolsConnection {
 public:
  virtual ~TargetPoolsConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  AddHealthCheck(google::cloud::cpp::compute::target_pools::v1::
                     AddHealthCheckRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> AddHealthCheck(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::target_pools::v1::
          AddHealthCheckRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  AddHealthCheck(ExperimentalTag,
                 google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  AddInstance(
      google::cloud::cpp::compute::target_pools::v1::AddInstanceRequest const&
          request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> AddInstance(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::target_pools::v1::AddInstanceRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  AddInstance(ExperimentalTag,
              google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StreamRange<std::pair<
      std::string, google::cloud::cpp::compute::v1::TargetPoolsScopedList>>
  AggregatedListTargetPools(google::cloud::cpp::compute::target_pools::v1::
                                AggregatedListTargetPoolsRequest request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  DeleteTargetPool(google::cloud::cpp::compute::target_pools::v1::
                       DeleteTargetPoolRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> DeleteTargetPool(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::target_pools::v1::
          DeleteTargetPoolRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  DeleteTargetPool(ExperimentalTag,
                   google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StatusOr<google::cloud::cpp::compute::v1::TargetPool> GetTargetPool(
      google::cloud::cpp::compute::target_pools::v1::GetTargetPoolRequest const&
          request);

  virtual StatusOr<google::cloud::cpp::compute::v1::TargetPoolInstanceHealth>
  GetHealth(
      google::cloud::cpp::compute::target_pools::v1::GetHealthRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  InsertTargetPool(google::cloud::cpp::compute::target_pools::v1::
                       InsertTargetPoolRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> InsertTargetPool(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::target_pools::v1::
          InsertTargetPoolRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  InsertTargetPool(ExperimentalTag,
                   google::cloud::cpp::compute::v1::Operation const& operation);

  virtual StreamRange<google::cloud::cpp::compute::v1::TargetPool>
  ListTargetPools(
      google::cloud::cpp::compute::target_pools::v1::ListTargetPoolsRequest
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  RemoveHealthCheck(google::cloud::cpp::compute::target_pools::v1::
                        RemoveHealthCheckRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  RemoveHealthCheck(ExperimentalTag, NoAwaitTag,
                    google::cloud::cpp::compute::target_pools::v1::
                        RemoveHealthCheckRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  RemoveHealthCheck(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  RemoveInstance(google::cloud::cpp::compute::target_pools::v1::
                     RemoveInstanceRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> RemoveInstance(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::target_pools::v1::
          RemoveInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  RemoveInstance(ExperimentalTag,
                 google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  SetBackup(
      google::cloud::cpp::compute::target_pools::v1::SetBackupRequest const&
          request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation> SetBackup(
      ExperimentalTag, NoAwaitTag,
      google::cloud::cpp::compute::target_pools::v1::SetBackupRequest const&
          request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  SetBackup(ExperimentalTag,
            google::cloud::cpp::compute::v1::Operation const& operation);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  SetSecurityPolicy(google::cloud::cpp::compute::target_pools::v1::
                        SetSecurityPolicyRequest const& request);

  virtual StatusOr<google::cloud::cpp::compute::v1::Operation>
  SetSecurityPolicy(ExperimentalTag, NoAwaitTag,
                    google::cloud::cpp::compute::target_pools::v1::
                        SetSecurityPolicyRequest const& request);

  virtual future<StatusOr<google::cloud::cpp::compute::v1::Operation>>
  SetSecurityPolicy(
      ExperimentalTag,
      google::cloud::cpp::compute::v1::Operation const& operation);
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace compute_target_pools_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_COMPUTE_TARGET_POOLS_V1_TARGET_POOLS_CONNECTION_H
