// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/networkmanagement/v1/reachability.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_NETWORKMANAGEMENT_V1_REACHABILITY_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_NETWORKMANAGEMENT_V1_REACHABILITY_CONNECTION_H

#include "google/cloud/networkmanagement/v1/internal/reachability_retry_traits.h"
#include "google/cloud/networkmanagement/v1/reachability_connection_idempotency_policy.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/experimental_tag.h"
#include "google/cloud/future.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/networkmanagement/v1/reachability.pb.h>
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace networkmanagement_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `ReachabilityServiceConnection`.
class ReachabilityServiceRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<ReachabilityServiceRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `ReachabilityServiceConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class ReachabilityServiceLimitedErrorCountRetryPolicy
    : public ReachabilityServiceRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit ReachabilityServiceLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  ReachabilityServiceLimitedErrorCountRetryPolicy(
      ReachabilityServiceLimitedErrorCountRetryPolicy&& rhs) noexcept
      : ReachabilityServiceLimitedErrorCountRetryPolicy(
            rhs.maximum_failures()) {}
  ReachabilityServiceLimitedErrorCountRetryPolicy(
      ReachabilityServiceLimitedErrorCountRetryPolicy const& rhs) noexcept
      : ReachabilityServiceLimitedErrorCountRetryPolicy(
            rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<ReachabilityServiceRetryPolicy> clone() const override {
    return std::make_unique<ReachabilityServiceLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = ReachabilityServiceRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      networkmanagement_v1_internal::ReachabilityServiceRetryTraits>
      impl_;
};

/**
 * A retry policy for `ReachabilityServiceConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class ReachabilityServiceLimitedTimeRetryPolicy
    : public ReachabilityServiceRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit ReachabilityServiceLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  ReachabilityServiceLimitedTimeRetryPolicy(
      ReachabilityServiceLimitedTimeRetryPolicy&& rhs) noexcept
      : ReachabilityServiceLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  ReachabilityServiceLimitedTimeRetryPolicy(
      ReachabilityServiceLimitedTimeRetryPolicy const& rhs) noexcept
      : ReachabilityServiceLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<ReachabilityServiceRetryPolicy> clone() const override {
    return std::make_unique<ReachabilityServiceLimitedTimeRetryPolicy>(
        maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = ReachabilityServiceRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      networkmanagement_v1_internal::ReachabilityServiceRetryTraits>
      impl_;
};

/**
 * The `ReachabilityServiceConnection` object for `ReachabilityServiceClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `ReachabilityServiceClient`. This allows users to inject custom
 * behavior (e.g., with a Google Mock object) when writing tests that use
 * objects of type `ReachabilityServiceClient`.
 *
 * To create a concrete instance, see `MakeReachabilityServiceConnection()`.
 *
 * For mocking, see
 * `networkmanagement_v1_mocks::MockReachabilityServiceConnection`.
 */
class ReachabilityServiceConnection {
 public:
  virtual ~ReachabilityServiceConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StreamRange<google::cloud::networkmanagement::v1::ConnectivityTest>
  ListConnectivityTests(
      google::cloud::networkmanagement::v1::ListConnectivityTestsRequest
          request);

  virtual StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>
  GetConnectivityTest(
      google::cloud::networkmanagement::v1::GetConnectivityTestRequest const&
          request);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>>
  CreateConnectivityTest(
      google::cloud::networkmanagement::v1::CreateConnectivityTestRequest const&
          request);

  virtual StatusOr<google::longrunning::Operation> CreateConnectivityTest(
      ExperimentalTag, NoAwaitTag,
      google::cloud::networkmanagement::v1::CreateConnectivityTestRequest const&
          request);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>>
  CreateConnectivityTest(ExperimentalTag,
                         google::longrunning::Operation const& operation);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>>
  UpdateConnectivityTest(
      google::cloud::networkmanagement::v1::UpdateConnectivityTestRequest const&
          request);

  virtual StatusOr<google::longrunning::Operation> UpdateConnectivityTest(
      ExperimentalTag, NoAwaitTag,
      google::cloud::networkmanagement::v1::UpdateConnectivityTestRequest const&
          request);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>>
  UpdateConnectivityTest(ExperimentalTag,
                         google::longrunning::Operation const& operation);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>>
  RerunConnectivityTest(
      google::cloud::networkmanagement::v1::RerunConnectivityTestRequest const&
          request);

  virtual StatusOr<google::longrunning::Operation> RerunConnectivityTest(
      ExperimentalTag, NoAwaitTag,
      google::cloud::networkmanagement::v1::RerunConnectivityTestRequest const&
          request);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::ConnectivityTest>>
  RerunConnectivityTest(ExperimentalTag,
                        google::longrunning::Operation const& operation);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::OperationMetadata>>
  DeleteConnectivityTest(
      google::cloud::networkmanagement::v1::DeleteConnectivityTestRequest const&
          request);

  virtual StatusOr<google::longrunning::Operation> DeleteConnectivityTest(
      ExperimentalTag, NoAwaitTag,
      google::cloud::networkmanagement::v1::DeleteConnectivityTestRequest const&
          request);

  virtual future<
      StatusOr<google::cloud::networkmanagement::v1::OperationMetadata>>
  DeleteConnectivityTest(ExperimentalTag,
                         google::longrunning::Operation const& operation);
};

/**
 * A factory function to construct an object of type
 * `ReachabilityServiceConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of
 * ReachabilityServiceClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `ReachabilityServiceConnection`. Expected options are any of the
 * types in the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::networkmanagement_v1::ReachabilityServicePolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `ReachabilityServiceConnection`
 * created by this function.
 */
std::shared_ptr<ReachabilityServiceConnection>
MakeReachabilityServiceConnection(Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace networkmanagement_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_NETWORKMANAGEMENT_V1_REACHABILITY_CONNECTION_H
