// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/video/transcoder/v1/services.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VIDEO_TRANSCODER_V1_TRANSCODER_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VIDEO_TRANSCODER_V1_TRANSCODER_CONNECTION_H

#include "google/cloud/video/transcoder/v1/internal/transcoder_retry_traits.h"
#include "google/cloud/video/transcoder/v1/transcoder_connection_idempotency_policy.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/experimental_tag.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/options.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/video/transcoder/v1/services.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace video_transcoder_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `TranscoderServiceConnection`.
class TranscoderServiceRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<TranscoderServiceRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `TranscoderServiceConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class TranscoderServiceLimitedErrorCountRetryPolicy
    : public TranscoderServiceRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit TranscoderServiceLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  TranscoderServiceLimitedErrorCountRetryPolicy(
      TranscoderServiceLimitedErrorCountRetryPolicy&& rhs) noexcept
      : TranscoderServiceLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}
  TranscoderServiceLimitedErrorCountRetryPolicy(
      TranscoderServiceLimitedErrorCountRetryPolicy const& rhs) noexcept
      : TranscoderServiceLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<TranscoderServiceRetryPolicy> clone() const override {
    return std::make_unique<TranscoderServiceLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = TranscoderServiceRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      video_transcoder_v1_internal::TranscoderServiceRetryTraits>
      impl_;
};

/**
 * A retry policy for `TranscoderServiceConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class TranscoderServiceLimitedTimeRetryPolicy
    : public TranscoderServiceRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit TranscoderServiceLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  TranscoderServiceLimitedTimeRetryPolicy(
      TranscoderServiceLimitedTimeRetryPolicy&& rhs) noexcept
      : TranscoderServiceLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  TranscoderServiceLimitedTimeRetryPolicy(
      TranscoderServiceLimitedTimeRetryPolicy const& rhs) noexcept
      : TranscoderServiceLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<TranscoderServiceRetryPolicy> clone() const override {
    return std::make_unique<TranscoderServiceLimitedTimeRetryPolicy>(
        maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = TranscoderServiceRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      video_transcoder_v1_internal::TranscoderServiceRetryTraits>
      impl_;
};

/**
 * The `TranscoderServiceConnection` object for `TranscoderServiceClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `TranscoderServiceClient`. This allows users to inject custom
 * behavior (e.g., with a Google Mock object) when writing tests that use
 * objects of type `TranscoderServiceClient`.
 *
 * To create a concrete instance, see `MakeTranscoderServiceConnection()`.
 *
 * For mocking, see
 * `video_transcoder_v1_mocks::MockTranscoderServiceConnection`.
 */
class TranscoderServiceConnection {
 public:
  virtual ~TranscoderServiceConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StatusOr<google::cloud::video::transcoder::v1::Job> CreateJob(
      google::cloud::video::transcoder::v1::CreateJobRequest const& request);

  virtual StreamRange<google::cloud::video::transcoder::v1::Job> ListJobs(
      google::cloud::video::transcoder::v1::ListJobsRequest request);

  virtual StatusOr<google::cloud::video::transcoder::v1::Job> GetJob(
      google::cloud::video::transcoder::v1::GetJobRequest const& request);

  virtual Status DeleteJob(
      google::cloud::video::transcoder::v1::DeleteJobRequest const& request);

  virtual StatusOr<google::cloud::video::transcoder::v1::JobTemplate>
  CreateJobTemplate(
      google::cloud::video::transcoder::v1::CreateJobTemplateRequest const&
          request);

  virtual StreamRange<google::cloud::video::transcoder::v1::JobTemplate>
  ListJobTemplates(
      google::cloud::video::transcoder::v1::ListJobTemplatesRequest request);

  virtual StatusOr<google::cloud::video::transcoder::v1::JobTemplate>
  GetJobTemplate(
      google::cloud::video::transcoder::v1::GetJobTemplateRequest const&
          request);

  virtual Status DeleteJobTemplate(
      google::cloud::video::transcoder::v1::DeleteJobTemplateRequest const&
          request);
};

/**
 * A factory function to construct an object of type
 * `TranscoderServiceConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of
 * TranscoderServiceClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `TranscoderServiceConnection`. Expected options are any of the types
 * in the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::video_transcoder_v1::TranscoderServicePolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `TranscoderServiceConnection` created
 * by this function.
 */
std::shared_ptr<TranscoderServiceConnection> MakeTranscoderServiceConnection(
    Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace video_transcoder_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VIDEO_TRANSCODER_V1_TRANSCODER_CONNECTION_H
